<?php

require_once("../mapviewerphp/constants.php");

try 
{
    MgInitializeWebTier(dirname(__FILE__)."\\..\\webconfig.ini");
}
catch(MgException $ex)
{
    echo $ex->GetDetails()."<br/>";
    echo $ex->GetStackTrace()."<br/>";
    die;
}

function CPPUNIT_ASSERT($condition, $msg = "") 
{
    if ($condition)
        echo "<p style='color: green'>Assertion pass: $msg</p>";
    else
        echo "<p style='color: red'>Assertion fail: $msg</p>";
}

function CPPUNIT_FAIL($msg = "") 
{
    echo "<p style='color: red'>General Failure: $msg</p>";
}

function FindClassByName($classDefCol, $name)
{
    for ($i =0; $i < $classDefCol->GetCount(); $i++)
    {
        $classDef = $classDefCol->GetItem($i);
        $temp = $classDef->GetName();
        if (strcmp($temp, $name) == 0)
            return $classDef;
    }
    return null;
}

try 
{
    $userInfo = new MgUserInformation("Administrator", "admin");
    $siteConn = new MgSiteConnection();
    $siteConn->Open($userInfo);
    
    $resource = new MgResourceIdentifier("Library://UnitTests/Data/Empty.FeatureSource");
    $resSvc = $siteConn->CreateService(MgServiceType::ResourceService);
    $bs = new MgByteSource(dirname(__FILE__)."\\Empty.FeatureSource");
    $br = $bs->GetReader();
    $resSvc->SetResource($resource, $br, null);
    $bs = new MgByteSource(dirname(__FILE__)."\\Empty.sdf");
    $br = $bs->GetReader();
    $resSvc->SetResourceData($resource, "Empty.sdf", "File", $br);

    $pService = $siteConn->CreateService(MgServiceType::FeatureService);

    //////////////////////////////////////////////////////////////////////
    /// The first test case:                                           ///
    /// Delete one property in an existing feature class               ///
    //////////////////////////////////////////////////////////////////////
    $oldSchemas = $pService->DescribeSchema($resource, "Schema", null);
    $oldSchema = $oldSchemas->GetItem(0);
    $oldSchemaClasses = $oldSchema->GetClasses();
    $oldClassDef = FindClassByName($oldSchemaClasses, "Parcel");
    $oldProperties = $oldClassDef->GetProperties();
    $oldPropDef = $oldProperties->GetItem("Name");
    $oldPropDef->Delete();
    $pService->ApplySchema($resource, $oldSchema);

    // Verify results
    $newSchemas = $pService->DescribeSchema($resource, "Schema", null);
    $newSchema = $newSchemas->GetItem(0);
    $newSchemaClasses = $newSchema->GetClasses();
    $newClassDef = FindClassByName($newSchemaClasses, "Parcel");
    $newProperties = $newClassDef->GetProperties();
    CPPUNIT_ASSERT($newProperties->Contains("Name") == false, 'newProperties->Contains("Name") == false');

    //////////////////////////////////////////////////////////////////////
    /// The second test case:                                          ///
    /// Add two feature classes                                        ///
    //////////////////////////////////////////////////////////////////////

    // Create the first class definition
    $classDef1 = new MgClassDefinition();
    $classDef1->SetName("FeatureClass1");
    $classDef1->SetDescription("Feature class 1");
    $classDef1->SetDefaultGeometryPropertyName("GEOM");

    $identityProperties1 = $classDef1->GetIdentityProperties();
    $properties1 = $classDef1->GetProperties();

    // Set key property
    $prop = new MgDataPropertyDefinition("KEY1");
    $prop->SetDataType(MgPropertyType::Int32);
    $prop->SetAutoGeneration(true);
    $prop->SetReadOnly(true);
    $identityProperties1->Add($prop);
    $properties1->Add($prop);

    // Set name property
    $prop = new MgDataPropertyDefinition("NAME1");
    $prop->SetDataType(MgPropertyType::String);
    $properties1->Add($prop);

    // Set geometry property
    $propGeom = new MgGeometricPropertyDefinition("GEOM");
    $propGeom->SetGeometryTypes(MgFeatureGeometricType::Surface);
    $properties1->Add($propGeom);

    // Create the second class definition
    $classDef2 = new MgClassDefinition();
    $classDef2->SetName("FeatureClass2");
    $classDef2->SetDescription("Feature class 2");
    $classDef2->SetDefaultGeometryPropertyName("GEOM");

    $identityProperties2 = $classDef2->GetIdentityProperties();
    $properties2 = $classDef2->GetProperties();

    // Set key property
    $prop = new MgDataPropertyDefinition("KEY2");
    $prop->SetDataType(MgPropertyType::Int32);
    $prop->SetAutoGeneration(true);
    $prop->SetReadOnly(true);
    $identityProperties2->Add($prop);
    $properties2->Add($prop);

    // Set name property
    $prop = new MgDataPropertyDefinition("NAME2");
    $prop->SetDataType(MgPropertyType::String);
    $properties2->Add($prop);

    // Set geometry property
    $propGeom = new MgGeometricPropertyDefinition("GEOM");
    $propGeom->SetGeometryTypes(MgFeatureGeometricType::Curve);
    $properties2->Add($propGeom);

    $schema = new MgFeatureSchema("Schema", "Schema");
    $schemaClasses = $schema->GetClasses();
    $schemaClasses->Add($classDef1);
    $schemaClasses->Add($classDef2);

    $pService->ApplySchema($resource, $schema);
    $newSchemas = $pService->DescribeSchema($resource, "Schema", null);

    // Verify schema
    $newSchema = $newSchemas->GetItem(0);
    $temp = $newSchema->GetName();
    CPPUNIT_ASSERT(strcmp($temp, "Schema") == 0, 'strcmp(temp, "Schema") == 0');

    $schemaClasses = $newSchema->GetClasses();

    // Verify the first feature class
    $lassDef1 = FindClassByName($schemaClasses, "FeatureClass1");
    $temp = $classDef1->GetName();
    CPPUNIT_ASSERT(strcmp($temp, "FeatureClass1") == 0, 'strcmp(temp, "FeatureClass1") == 0');
    $temp = $classDef1->GetDescription();
    CPPUNIT_ASSERT(strcmp($temp, "Feature class 1") == 0, 'strcmp(temp, "Feature class 1") == 0');
    $temp = $classDef1->GetDefaultGeometryPropertyName();
    CPPUNIT_ASSERT(strcmp($temp, "GEOM") == 0, 'strcmp(temp, "GEOM") == 0');

    $identityProperties1 = $classDef1->GetIdentityProperties();
    CPPUNIT_ASSERT($identityProperties1->GetCount() == 1, "identityProperties1->GetCount() == 1");
    $prop = ($identityProperties1->GetItem("KEY1"));
    CPPUNIT_ASSERT($prop != null, "prop != null");

    $properties1 = $classDef1->GetProperties();
    CPPUNIT_ASSERT($properties1->GetCount() == 3, "properties1->GetCount() == 3");

    $prop = ($properties1->GetItem("KEY1"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::Int32, "prop->GetDataType() == MgPropertyType::Int32");
    CPPUNIT_ASSERT($prop->IsAutoGenerated() == true, "prop->IsAutoGenerated() == true");
    CPPUNIT_ASSERT($prop->GetReadOnly() == true, "prop->GetReadOnly() == true");

    $prop = ($properties1->GetItem("NAME1"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::String, "prop->GetDataType() == MgPropertyType::String");

    $propGeom = ($properties1->GetItem("GEOM"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($propGeom->GetGeometryTypes() == MgFeatureGeometricType::Surface, "propGeom->GetGeometryTypes() == MgFeatureGeometricType::Surface");

    // Verify the second feature class
    $classDef2 = FindClassByName($schemaClasses, "FeatureClass2");
    $temp = $classDef2->GetDescription();
    CPPUNIT_ASSERT(strcmp($temp, "Feature class 2") == 0);
    $temp = $classDef2->GetDefaultGeometryPropertyName();
    CPPUNIT_ASSERT(strcmp($temp, "GEOM") == 0, 'strcmp(temp, "GEOM") == 0');

    $identityProperties2 = $classDef2->GetIdentityProperties();
    CPPUNIT_ASSERT($identityProperties2->GetCount() == 1);
    $prop = ($identityProperties2->GetItem("KEY2"));
    CPPUNIT_ASSERT($prop != null, "prop != null");

    $properties2 = $classDef2->GetProperties();
    CPPUNIT_ASSERT($properties2->GetCount() == 3, "properties2->GetCount() == 3");

    $prop = ($properties2->GetItem("KEY2"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::Int32, "prop->GetDataType() == MgPropertyType::Int32");
    CPPUNIT_ASSERT($prop->IsAutoGenerated() == true, "prop->IsAutoGenerated() == true");
    CPPUNIT_ASSERT($prop->GetReadOnly() == true, "prop->GetReadOnly() == true");

    $prop = ($properties2->GetItem("NAME2"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::String, "prop->GetDataType() == MgPropertyType::String");

    $propGeom = ($properties2->GetItem("GEOM"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($propGeom->GetGeometryTypes() == MgFeatureGeometricType::Curve, "propGeom->GetGeometryTypes() == MgFeatureGeometricType::Curve");

    //////////////////////////////////////////////////////////////////////
    /// The third test case:                                           ///
    /// Delete one feature class and modify one feature class          ///
    //////////////////////////////////////////////////////////////////////
    // Delete the first feature class
    $classDef1 = FindClassByName($schemaClasses, "FeatureClass1");
    $classDef1->Delete();

    // Modify the second feature class
    $classDef2 = FindClassByName($schemaClasses, "FeatureClass2");
    $classDef2->SetDescription("Modified Feature Class");
    $properties2 = $classDef2->GetProperties();

    $prop = new MgDataPropertyDefinition("LENGTH");
    $prop->SetDataType(MgPropertyType::Single);
    $properties2->Add($prop);

    $prop = new MgDataPropertyDefinition("ID");
    $prop->SetDataType(MgPropertyType::Int32);
    $prop->SetAutoGeneration(true);
    $prop->SetReadOnly(true);
    $properties2->Add($prop);

    $pService->ApplySchema($resource, $newSchema);

    $newSchemas = $pService->DescribeSchema($resource, "Schema", null);
    $newSchema = $newSchemas->GetItem(0);
    $temp = $newSchema->GetName();
    CPPUNIT_ASSERT(strcmp($temp, "Schema") == 0, 'strcmp(temp, "Schema") == 0');

    $schemaClasses = $newSchema->GetClasses();

    // Verify the first feature class
    $classDef1 = FindClassByName($schemaClasses, "FeatureClass1");
    CPPUNIT_ASSERT($classDef1 == null, "classDef1 == null");

    $classDef2 = FindClassByName($schemaClasses, "FeatureClass2");
    $temp = $classDef2->GetDescription();
    CPPUNIT_ASSERT(strcmp($temp, "Modified Feature Class") == 0, 'strcmp(temp, "Modified Feature Class") == 0');

    $properties2 = $classDef2->GetProperties();
    CPPUNIT_ASSERT($properties2->GetCount() == 5, "properties2->GetCount() == 5");

    $temp = $classDef2->GetDefaultGeometryPropertyName();
    CPPUNIT_ASSERT(strcmp($temp, "GEOM") == 0, "strcmp(temp, 'GEOM') == 0");

    $identityProperties2 = $classDef2->GetIdentityProperties();
    CPPUNIT_ASSERT($identityProperties2->GetCount() == 1, "identityProperties2->GetCount() == 1");
    $prop = ($identityProperties2->GetItem("KEY2"));
    CPPUNIT_ASSERT($prop != null, "prop != null");

    $prop = ($properties2->GetItem("ID"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::Int32, "prop->GetDataType() == MgPropertyType::Int32");
    CPPUNIT_ASSERT($prop->IsAutoGenerated() == true, "prop->IsAutoGenerated() == true");
    CPPUNIT_ASSERT($prop->GetReadOnly() == true, "prop->GetReadOnly() == true");

    $prop = ($properties2->GetItem("LENGTH"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($prop->GetDataType() == MgPropertyType::Single, "prop->GetDataType() == MgPropertyType::Single");

    $propGeom = ($properties2->GetItem("GEOM"));
    CPPUNIT_ASSERT($prop != null, "prop != null");
    CPPUNIT_ASSERT($propGeom->GetGeometryTypes() == MgFeatureGeometricType::Curve, "propGeom->GetGeometryTypes() == MgFeatureGeometricType::Curve");
}
catch(MgException $e)
{
    CPPUNIT_FAIL($e->GetExceptionMessage());
}

?>